:- set(clauselength,5).
:- set(min_coverage,40).

:- modeh(1,active(+drug)).

% atom - substructure of drug

:- modeb(10,atm(+drug,-atomid)).

% single atom properties

:- modeb(1,atm_elm(+atomid,-element)).
:- modeb(1,atm_int(+atomid,-integer)).
:- modeb(1,atm_chr(+atomid,-charge)).

:- modeb(*,instantiate(+element)).
:- modeb(*,instantiate(+integer)).
:- modeb(*,instantiate(+charge)).
:- modeb(1,lteq_c(+charge,-real)).
:- modeb(1,gteq_c(+charge,-real)).

% bond property

:- modeb(1,atm_bond(+atomid,+atomid,-integer)).
 :- modeb(*,instantiate(+integer)). % declared above

% drug properties 

:- modeb(1,ind1(+drug,-real)).
:- modeb(1,inda(+drug,-real)).

:- modeb(1,lumo(+drug,-energy)).
:- modeb(1,lteq_e(+energy,-real)).
:- modeb(1,gteq_e(+energy,-real)).
:- modeb(*,instantiate(+energy)).

:- modeb(1,logp(+drug,-hydrophob)).
:- modeb(1,lteq_h(+hydrophob,-real)).
:- modeb(1,gteq_h(+hydrophob,-real)).
:- modeb(*,instantiate(+hydrophob)).

:- modeb(1,hansch_attr(+drug,-hval)).
:- modeb(1,lteq_hv(+hval,-real)).
:- modeb(1,gteq_hv(+hval,-real)).
:- modeb(*,instantiate(+hval)).

:- modeb(*,instantiate(+real)).

:- modeb(10,benzene(+drug,-ring)).
:- modeb(10,carbon_5_aromatic_ring(+drug,-ring)).
:- modeb(10,carbon_6_ring(+drug,-ring)).
:- modeb(10,hetero_aromatic_6_ring(+drug,-ring)).
:- modeb(10,hetero_aromatic_5_ring(+drug,-ring)).
:- modeb(10,ring_size_6(+drug,-ring)).
:- modeb(10,ring_size_5(+drug,-ring)).
:- modeb(10,nitro(+drug,-ring)).
:- modeb(10,methyl(+drug,-ring)).

:- modeb(1,connected(+ring,+ring)).

:- modeb(10,anthracene(+drug,-ringlist)).
:- modeb(10,phenanthrene(+drug,-ringlist)).
:- modeb(10,ball3(+drug,-ringlist)).

:- modeb(1,member(+ring,+ringlist)).

%:- modeb(10,dist(+drug,-group,-group,-group,-group,-real,-real)).

:- modeb(10,dist(+drug,-dist_desc)).
:- modeb(10,instantiate(+dist_desc)).
*/

% background knowledge 

:- consult(types).

:- consult(lumo).
:- consult(logp).
:- consult(hansch).
:- consult(ind1).
:- consult(inda).

:- consult(atom_bond).
:- consult(two_dim).
:- consult(three_dim).


% other background predicates and utilities

atm(A,B):-
    atm(A,B,_,_,_).

atm_elm(A,B):-
    atm(_,A,B,_,_).
atm_int(A,B):-
    atm(_,A,_,B,_).
atm_chr(A,B):-
    atm(_,A,_,_,B).

atm_bond(A,B,C):-
    bond(_,A,B,C).

dist_group(A,(C,E,F,G)):-
    dist(A,B,C,D,E,F,G).

gteq_e(X,Y):-
    real_e(Y), 
    X >= Y, !.

lteq_e(X,Y):-
    real_e(Y),
    X =< Y, !.

gteq_c(X,Y):-
    real_c(Y), 
    X >= Y, !.

lteq_c(X,Y):-
    real_c(Y),
    X =< Y, !.

gteq_h(X,Y):-
    real_h(Y), 
    X >= Y, !.

lteq_h(X,Y):-
    real_h(Y),
    X =< Y, !.

gteq_hv(X,Y):-
    real_hv(Y), 
    X >= Y, !.

lteq_hv(X,Y):-
    real_hv(Y),
    X =< Y, !.

real_e(X):-
    lumo(_,X).

real_c(X):-
    atm(_,_,_,_,X).

real_h(X):-
    logp(_,X).

real_hv(X):-
    hansch_attr(_,X).

member(X,[X|_]).
member(X,[_|T]):-
    member(X,T).

connected(Ring1,Ring2):-
    Ring1 \= Ring2,
    member(Atom,Ring1),
    member(Atom,Ring2), !.


symbond(D,A1,A2,Type):-
    bond(D,A1,A2,Type).
symbond(D,A1,A2,Type):-
    bond(D,A2,A1,Type).

dist(D,A1,G1,A2,G2,Dist,Error):-
    var(G1), var(G2), !,
    group(D,G1,A1,X1,Y1,Z1),
    group(D,G2,A2,X2,Y2,Z2),
    G2 @>= G1,
    not(identical(G1,A1,G2,A2)),
        euc_dist(p(X1,Y1,Z1),p(X2,Y2,Z2),Dist),
        error(Error).
dist(D,A1,G1,A2,G2,Dist,Error):-
    group(D,G1,A1,X1,Y1,Z1),
    group(D,G2,A2,X2,Y2,Z2),
    not(identical(G1,A1,G2,A2)),
        euc_dist(p(X1,Y1,Z1),p(X2,Y2,Z2),Dist1),
    Diff is Dist1 - Dist,
        absolute_value(Diff,E1),
        E1 =< Error.
 
absolute_value(X,X):- X >= 0, !.
absolute_value(X,Y):- X < 0, Y is -X.

identical(G,A1,G,A2):-
    sort(A1,A1s), sort(A2,A2s),
    A1s == A2s.

euc_dist(p(X1,Y1,Z1),p(X2,Y2,Z2),D):-
        Dsq is (X1 - X2)^2 + (Y1 - Y2)^2 + (Z1 - Z2)^2,
        D is sqrt(Dsq).
